@@ -1,20 +1,20 @@
 #' Confidence Intervals for the Indirect Effect
 #'
-#' This function computes confidence intervals for the indirect effect based on the
-#' asymptotic normal method, distribution of the product method and the Monte Carlo
-#' method. By default, the function uses the distribution of the product method
-#' for computing the two-sided 95\% asymmetric confidence intervals for the indirect
-#' effect product of coefficient estimator \eqn{\hat{a}\hat{b}}.
+#' This function computes confidence intervals for the indirect effect based on
+#' the asymptotic normal method, distribution of the product method and the Monte
+#' Carlo method. By default, the function uses the distribution of the product
+#' method for computing the two-sided 95\% asymmetric confidence intervals for
+#' the indirect effect product of coefficient estimator \eqn{\hat{a}\hat{b}}.
 #'
-#' In statistical mediation analysis (MacKinnon & Tofighi, 2013), the indirect effect
-#' refers to the effect of the independent variable \eqn{X} on the outcome variable
-#' \eqn{Y} transmitted by the mediator variable \eqn{M}. The magnitude of the indirect
-#' effect \eqn{ab} is quantified by the product of the the coefficient \eqn{a}
-#' (i.e., effect of \eqn{X} on \eqn{M}) and the coefficient \eqn{b} (i.e., effect of
-#' \eqn{M} on \eqn{Y} adjusted for \eqn{X}). In practice, researchers are often
-#' interested in confidence limit estimation for the indirect effect. This function
-#' offers three different methods for computing the confidence interval for the
-#' product of coefficient estimator \eqn{\hat{a}\hat{b}}:
+#' In statistical mediation analysis (MacKinnon & Tofighi, 2013), the indirect
+#' effect refers to the effect of the independent variable \eqn{X} on the outcome
+#' variable \eqn{Y} transmitted by the mediator variable \eqn{M}. The magnitude
+#' of the indirect effect \eqn{ab} is quantified by the product of the the
+#' coefficient \eqn{a} (i.e., effect of \eqn{X} on \eqn{M}) and the coefficient
+#' \eqn{b} (i.e., effect of \eqn{M} on \eqn{Y} adjusted for \eqn{X}). In practice,
+#' researchers are often interested in confidence limit estimation for the indirect
+#' effect. This function offers three different methods for computing the confidence
+#' interval for the product of coefficient estimator \eqn{\hat{a}\hat{b}}:
 #'
 #' \strong{(1) Asymptotic normal method}
 #'
@@ -42,13 +42,14 @@
 #'
 #' The asymptotic normal method is known to have low statistical power because
 #' the distribution of the product \eqn{\hat{a}\hat{b}} is not normally distributed.
-#' (Kisbu-Sakarya, MacKinnon, & Miocevic, 2014). In the null case, where both random
-#' variables have mean equal to zero, the distribution is symmetric with kurtosis of
-#' six. When the product of the means of the two random variables is nonzero, the
-#' distribution is skewed (up to a maximum value of \eqn{\pm} 1.5) and has a excess
-#' kurtosis (up to a maximum value of 6). However, the product approaches a normal
-#' distribution as one or both of the ratios of the means to standard errors of each
-#' random variable get large in absolute value (MacKinnon, Lockwood & Williams, 2004).
+#' (Kisbu-Sakarya, MacKinnon, & Miocevic, 2014). In the null case, where both
+#' random variables have mean equal to zero, the distribution is symmetric with
+#' kurtosis of six. When the product of the means of the two random variables is
+#' nonzero, the distribution is skewed (up to a maximum value of \eqn{\pm} 1.5)
+#' and has a excess kurtosis (up to a maximum value of 6). However, the product
+#' approaches a normal distribution as one or both of the ratios of the means to
+#' standard errors of each random variable get large in absolute value (MacKinnon,
+#' Lockwood & Williams, 2004).
 #'
 #' \strong{(2) Distribution of the product method}
 #'
@@ -58,23 +59,25 @@
 #' coefficients to compute \eqn{ab} and then uses the critical values for the
 #' distribution of the product (Meeker, Cornwell, & Aroian, 1981) to create
 #' asymmetric confidence intervals. The distribution of the product approaches
-#' the gamma distribution (Aroian, 1947). The analytical solution for the distribution
-#' of the product is provided by the Bessel function used to the solution of
-#' differential equations and is approximately proportional to the Bessel function
-#' of the second kind with a purely imaginary argument (Craig, 1936).
+#' the gamma distribution (Aroian, 1947). The analytical solution for the
+#' distribution of the product is provided by the Bessel function used to the
+#' solution of differential equations and is approximately proportional to the
+#' Bessel function of the second kind with a purely imaginary argument (Craig,
+#' 1936).
 #'
 #' \strong{(3) Monte Carlo method}
 #'
 #' The Monte Carlo (MC) method (MacKinnon et al., 2004) relies on the assumption
-#' that the parameters \eqn{a} and \eqn{b} have a joint normal sampling distribution.
-#' Based on the parametric assumption, a sampling distribution of the product
-#' \eqn{a}\eqn{b} using random samples with population values equal to the sample
-#' estimates \eqn{\hat{a}}, \eqn{\hat{b}}, \eqn{\hat{\sigma}_a}, and \eqn{\hat{\sigma}_b}
-#' is generated. Percentiles of the sampling distribution are identified to serve as
-#' limits for a \eqn{100(1 - \alpha)}\% asymmetric confidence interval about the sample
-#' \eqn{\hat{a}\hat{b}} (Preacher & Selig, 2012). Note that parametric assumptions
-#' are invoked for \eqn{\hat{a}} and \eqn{\hat{b}}, but no parametric assumptions
-#' are made about the distribution of \eqn{\hat{a}\hat{b}}.
+#' that the parameters \eqn{a} and \eqn{b} have a joint normal sampling
+#' distribution. Based on the parametric assumption, a sampling distribution of
+#' the product \eqn{a}\eqn{b} using random samples with population values equal
+#' to the sample estimates \eqn{\hat{a}}, \eqn{\hat{b}}, \eqn{\hat{\sigma}_a},
+#' and \eqn{\hat{\sigma}_b} is generated. Percentiles of the sampling distribution
+#' are identified to serve as limits for a \eqn{100(1 - \alpha)}\% asymmetric
+#' confidence interval about the sample \eqn{\hat{a}\hat{b}} (Preacher & Selig,
+#' 2012). Note that parametric assumptions are invoked for \eqn{\hat{a}} and
+#' \eqn{\hat{b}}, but no parametric assumptions are made about the distribution
+#' of \eqn{\hat{a}\hat{b}}.
 #'
 #' @param a           a numeric value indicating the coefficient \eqn{a}, i.e.,
 #'                    effect of \eqn{X} on \eqn{M}.
@@ -84,31 +87,41 @@
 #'                    \eqn{a}.
 #' @param se.b        a positive numeric value indicating the standard error of
 #'                    \eqn{b}.
-#' @param print       a character string or character vector indicating which confidence
-#'                    intervals (CI) to show on the console, i.e. \code{"all"} for all
-#'                    CIs, \code{"asymp"} for the CI based on the asymptotic normal
-#'                    method, \code{"dop"} (default) for the CI based on the distribution
-#'                    of the product method, and \code{"mc"} for the CI based on the Monte
-#'                    Carlo method.
-#' @param se          a character string indicating which standard error (SE) to compute
-#'                    for the asymptotic normal method, i.e., \code{"sobel"} for the
-#'                    approximate standard error by Sobel (1982) using the multivariate
-#'                    delta method based on a first order Taylor series approximation,
-#'                    \code{"aroian"} (default) for the exact standard error by
-#'                    Aroian (1947) based on a first and second order Taylor series
-#'                    approximation, and \code{"goodman"} for the unbiased standard
-#'                    error by Goodman (1960).
+#' @param print       a character string or character vector indicating which
+#'                    confidence intervals (CI) to show on the console, i.e.
+#'                    \code{"all"} for all CIs, \code{"asymp"} for the CI based
+#'                    on the asymptotic normal method, \code{"dop"} (default) for
+#'                    the CI based on the distribution of the product method, and
+#'                    \code{"mc"} for the CI based on the Monte Carlo method.
+#' @param se          a character string indicating which standard error (SE) to
+#'                    compute for the asymptotic normal method, i.e., \code{"sobel"}
+#'                    for the approximate standard error by Sobel (1982) using the
+#'                    multivariate delta method based on a first order Taylor series
+#'                    approximation, \code{"aroian"} (default) for the exact standard
+#'                    error by Aroian (1947) based on a first and second order Taylor
+#'                    series approximation, and \code{"goodman"} for the unbiased
+#'                    standard error by Goodman (1960).
 #' @param nrep        an integer value indicating the number of Monte Carlo repetitions.
-#' @param alternative a character string specifying the alternative hypothesis, must be
-#'                    one of \code{"two.sided"} (default), \code{"greater"} or \code{"less"}.
-#' @param seed        a numeric value specifying the seed of the random number generator
-#'                    when using the Monte Carlo method.
-#' @param conf.level  a numeric value between 0 and 1 indicating the confidence level
-#'                    of the interval.
+#' @param alternative a character string specifying the alternative hypothesis,
+#'                    must be one of \code{"two.sided"} (default), \code{"greater"}
+#'                    or \code{"less"}.
+#' @param seed        a numeric value specifying the seed of the random number
+#'                    generator when using the Monte Carlo method.
+#' @param conf.level  a numeric value between 0 and 1 indicating the confidence
+#'                    level of the interval.
 #' @param digits      an integer value indicating the number of decimal places
 #'                    to be used for displaying
-#' @param check       logical: if \code{TRUE}, argument specification is checked.
-#' @param output      logical: if \code{TRUE}, output is shown on the console.
+#' @param write       a character string naming a text file with file extension
+#'                    \code{".txt"} (e.g., \code{"Output.txt"}) for writing the
+#'                    output into a text file.
+#' @param append      logical: if \code{TRUE} (default), output will be appended
+#'                    to an existing text file with extension \code{.txt} specified
+#'                    in \code{write}, if \code{FALSE} existing text file will be
+#'                    overwritten.
+#' @param check        logical: if \code{TRUE} (default), argument specification
+#'                    is checked.
+#' @param output      logical: if \code{TRUE} (default), output is shown on the
+#'                    console.
 #'
 #' @author
 #' Takuya Yanagida \email{takuya.yanagida@@univie.ac.at}
@@ -175,18 +188,24 @@
 #' @export
 #'
 #' @examples
-#' # Distribution of the Product Method
+#' # Example 1: Distribution of the Product Method
 #' indirect(a = 0.35, b = 0.27, se.a = 0.12, se.b = 0.18)
 #'
-#' # Monte Carlo Method
+#' # Example 2: Monte Carlo Method
 #' indirect(a = 0.35, b = 0.27, se.a = 0.12, se.b = 0.18, print = "mc")
 #'
-#' # Asymptotic Normal Method
+#' # Example 3: Asymptotic Normal Method
 #' indirect(a = 0.35, b = 0.27, se.a = 0.12, se.b = 0.18, print = "asymp")
+#'
+#' \dontrun{
+#' # Example 4: Write Results into a text file
+#' indirect(a = 0.35, b = 0.27, se.a = 0.12, se.b = 0.18, write = "Indirect.txt")
+#' }
 indirect <- function(a, b, se.a, se.b, print = c("all", "asymp", "dop", "mc"),
                      se = c("sobel", "aroian", "goodman"), nrep = 100000,
                      alternative = c("two.sided", "less", "greater"), seed = NULL,
-                     conf.level = 0.95, digits = 3, check = TRUE, output = TRUE) {
+                     conf.level = 0.95, digits = 3, write = NULL, append = TRUE,
+                     check = TRUE, output = TRUE) {
 
   #_____________________________________________________________________________
   #
@@ -227,6 +246,12 @@ indirect <- function(a, b, se.a, se.b, print = c("all", "asymp", "dop", "mc"),
     # Check input 'digits'
     if (isTRUE(digits %% 1L != 0L || digits < 0L)) { stop("Specify a positive integer number for the argument 'digits'.", call. = FALSE) }
 
+    # Check input 'write'
+    if (isTRUE(!is.null(write) && substr(write, nchar(write) - 3L, nchar(write)) != ".txt")) { stop("Please specify a character string with file extenstion '.txt' for the argument 'write'.") }
+
+    # Check input 'append'
+    if (isTRUE(!is.logical(append))) { stop("Please specify TRUE or FALSE for the argument 'append'.", call. = FALSE) }
+
     # Check input 'output'
     if (isTRUE(!is.logical(output))) { stop("Please specify TRUE or FALSE for the argument 'output'.", call. = FALSE) }
 
@@ -489,7 +514,7 @@ indirect <- function(a, b, se.a, se.b, print = c("all", "asymp", "dop", "mc"),
                  data = list(a = a, b = b, se.a = se.b, se.b = se.b),
                  args = list(print = print, se = se, nrep = nrep, alternative = alternative,
                              seed = seed, conf.level = conf.level, digits = digits,
-                             check = check, output = output),
+                             write = write, append = append, check = check, output = output),
                  result = list(asymp = data.frame(est = asymp.ab, se = asmyp.se.ab,
                                                   low = asymp.ci[1L], upp = asymp.ci[2L], row.names = NULL),
                                dop = data.frame(est = dop.ab, se = dop.se.ab,
@@ -499,6 +524,32 @@ indirect <- function(a, b, se.a, se.b, print = c("all", "asymp", "dop", "mc"),
 
   class(object) <- "misty.object"
 
+  #_____________________________________________________________________________
+  #
+  # Write Results --------------------------------------------------------------
+
+  #_____________________________________________________________________________
+  #
+  # Write Results --------------------------------------------------------------
+
+  if (isTRUE(!is.null(write))) {
+
+    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+    ## Text file ####
+
+    # Send R output to textfile
+    sink(file = write, append = ifelse(isTRUE(file.exists(write)), append, FALSE), type = "output", split = FALSE)
+
+    if (append && isTRUE(file.exists(write))) { write("", file = write, append = TRUE) }
+
+    # Print object
+    print(object, check = FALSE)
+
+    # Close file connection
+    sink()
+
+  }
+
   #_____________________________________________________________________________
   #
   # Output ---------------------------------------------------------------------
