@@ -1,65 +1,75 @@
 #' Read SPSS File
 #'
 #' This function calls the \code{read_spss} function in the \pkg{haven} package
-#' by Hadley Wickham and Evan Miller (2019) to read an SPSS file.
+#' by Hadley Wickham, Evan Miller and Danny Smith (2023) to read an SPSS file.
 #'
-#' @param file             a character string indicating the name of the SPSS data file
-#'                         with or without file extension '.sav', e.g., \code{"My_SPSS_Data.sav"}
-#'                         or \code{"My_SPSS_Data"}.
-#' @param use.value.labels logical: if \code{TRUE}, variables with value labels are converted into factors.
-#' @param use.missings     logical: if \code{TRUE} (default), user-defined missing values are converted into NAs.
-#' @param formats          logical: if \code{TRUE}, variable formats are shown in an attribute for all variables.
-#' @param label            logical: if \code{TRUE} (default), variable labels are shown in an attribute for all variables.
-#' @param labels           logical: if \code{TRUE} (default), value labels are shown in an attribute for all variables.
-#' @param missing          logical: if \code{TRUE}, value labels for user-defined missings are shown in an attribute.
+#' @param file             a character string indicating the name of the SPSS
+#'                         data file with or without file extension '.sav', e.g.,
+#'                         \code{"SPSS_Data.sav"} or \code{"SPSS_Data"}.
+#' @param use.value.labels logical: if \code{TRUE}, variables with value labels
+#'                         are converted into factors.
+#' @param use.missings     logical: if \code{TRUE} (default), user-defined missing
+#'                         values are converted into NAs.
+#' @param formats          logical: if \code{TRUE}, variable formats are shown in
+#'                         an attribute for all variables.
+#' @param label            logical: if \code{TRUE}, variable labels are
+#'                         shown in an attribute for all variables.
+#' @param labels           logical: if \code{TRUE}, value labels are
+#'                         shown in an attribute for all variables.
+#' @param missing          logical: if \code{TRUE}, value labels for user-defined
+#'                         missings are shown in an attribute for all variables.
+#' @param widths           logical: if \code{TRUE}, widths are shown in an attribute
 #'                         for all variables.
-#' @param widths           logical: if \code{TRUE}, widths are shown in an attribute for all variables.
-#' @param as.data.frame    logical: if \code{TRUE} (default), function returns a regular data frame (default);
+#' @param as.data.frame    logical: if \code{TRUE} (default), function returns a
+#'                         regular data frame;
 #'                         if \code{FALSE} function returns a tibble.
-#' @param check            logical: if \code{TRUE}, argument specification is checked.
-#'
+#' @param check            logical: if \code{TRUE} (default), argument specification
+#'                         is checked.
 #'
 #' @author
-#' Hadley Wickham and Evan Miller
+#' Hadley Wickham, Evan Miller and Danny Smith
 #'
 #' @seealso
-#' \code{\link{write.sav}}, \code{\link{read.xlsx}}, \code{\link{read.mplus}}
+#' \code{\link{read.dta}}, \code{\link{write.dta}}, \code{\link{read.xlsx}},
+#' \code{\link{write.xlsx}}, \code{\link{read.mplus}}, \code{\link{write.mplus}}
 #'
 #' @references
-#' Hadley Wickham and Evan Miller (2019). \emph{haven: Import and Export 'SPSS', 'Stata' and 'SAS' Files}.
-#' R package version 2.1.1.\url{https://CRAN.R-project.org/package=haven}
+#' Wickham H, Miller E, Smith D (2023). \emph{haven: Import and Export 'SPSS',
+#' 'Stata' and 'SAS' Files}. R package version 2.5.3.
+#' \url{https://CRAN.R-project.org/package=haven}
 #'
 #' @return
 #' Returns a data frame or tibble.
 #'
+#' @note
+#' This function is a modified copy of the \code{read_sav()} function in the
+#' \pkg{haven} package by Hadley Wickham, Evan Miller and Danny Smith (2023).
+#'
 #' @export
 #'
 #' @examples
 #' \dontrun{
-#' # Read SPSS data
+#' # Example 1: Read SPSS data file
 #' read.sav("SPSS_Data.sav")
 #' read.sav("SPSS_Data")
 #'
-#' # Read SPSS data, convert variables with value labels into factors
+#' # Example 2: Read SPSS data file, convert variables with value labels into factors
 #' read.sav("SPSS_Data.sav", use.value.labels = TRUE)
 #'
-#' # Read SPSS data, user-defined missing values are not converted into NAs
+#' # Example 3: Read SPSS data file, user-defined missing values are not converted into NAs
 #' read.sav("SPSS_Data.sav", use.missing = FALSE)
 #'
-#' # Read SPSS data as tibble
+#' # Example 4: Read SPSS data file as tibble
 #' read.sav("SPSS_Data.sav", as.data.frame = FALSE)
 #' }
 read.sav <- function(file, use.value.labels = FALSE, use.missings = TRUE, formats = FALSE,
-                     label = TRUE, labels = TRUE, missing = FALSE, widths = FALSE,
+                     label = FALSE, labels = FALSE, missing = FALSE, widths = FALSE,
                      as.data.frame = TRUE, check = TRUE) {
 
   #_____________________________________________________________________________
   #
   # Initial Check --------------------------------------------------------------
 
-  # Package haven installed?
-  if (isTRUE(!requireNamespace("haven", quietly = TRUE))) { stop("Package \"haven\" is needed for this function to work, please install it.", call. = FALSE ) }
-
   # Check input 'file'
   if (isTRUE(missing(file))) { stop("Please specify a character string indicating the name of the SPSS data file for the argument 'file'", call. = FALSE) }
 
@@ -69,15 +79,18 @@ read.sav <- function(file, use.value.labels = FALSE, use.missings = TRUE, format
   # Check if 'file' exists
   if (isTRUE(!file.exists(file))) { stop(paste0("Unable to open SPSS data file: ", sQuote(file), " does not exist."), call. = FALSE) }
 
-  # Check input 'check',
-  if (isTRUE(!is.logical(check))) { stop("Please specify TRUE or FALSE for the argument 'check'.", call. = FALSE) }
-
   #_____________________________________________________________________________
   #
   # Input Check ----------------------------------------------------------------
 
+  # Check input 'check',
+  if (isTRUE(!is.logical(check))) { stop("Please specify TRUE or FALSE for the argument 'check'.", call. = FALSE) }
+
   if (isTRUE(check)) {
 
+    # Package haven installed?
+    if (isTRUE(!requireNamespace("haven", quietly = TRUE))) { stop("Package \"haven\" is needed for this function to work, please install it.", call. = FALSE ) }
+
     # Check input 'use.value.labels'
     if (isTRUE(!is.logical(use.value.labels))) { stop("Please specify TRUE or FALSE for the argument 'use.value.labels'.", call. = FALSE) }
 
@@ -111,6 +124,9 @@ read.sav <- function(file, use.value.labels = FALSE, use.missings = TRUE, format
   # User-defined missing values
   use_na <- ifelse(isTRUE(use.missings), FALSE, TRUE)
 
+  # Value labels
+  labels <- ifelse(isTRUE(use.value.labels), TRUE, labels)
+
   #_____________________________________________________________________________
   #
   # Main Function --------------------------------------------------------------
@@ -121,7 +137,7 @@ read.sav <- function(file, use.value.labels = FALSE, use.missings = TRUE, format
   #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
   ## Remove format ####
 
-  if (!isTRUE(formats)) {
+  if (isTRUE(!formats)) {
 
     for (i in names(object)) {
 
@@ -132,9 +148,9 @@ read.sav <- function(file, use.value.labels = FALSE, use.missings = TRUE, format
   }
 
   #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-  ## Remove label ####
+  ## Remove Variable Label ####
 
-  if (!isTRUE(label)) {
+  if (isTRUE(!label)) {
 
     for (i in names(object)) {
 
@@ -144,10 +160,23 @@ read.sav <- function(file, use.value.labels = FALSE, use.missings = TRUE, format
 
   }
 
+  #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+  ## Remove Value Labels ####
+
+  if (isTRUE(!labels)) {
+
+    for (i in names(object)) {
+
+      object[, i] <- haven::zap_labels(object[, i])
+
+    }
+
+  }
+
   #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
   ## Remove value labels for user-defined missing values ####
 
-  if (!isTRUE(missing)) {
+  if (isTRUE(!missing)) {
 
     for (i in names(object)) {
 
@@ -160,7 +189,7 @@ read.sav <- function(file, use.value.labels = FALSE, use.missings = TRUE, format
   #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
   ## Remove widths ####
 
-  if (!isTRUE(widths)) {
+  if (!isTRUE(!widths)) {
 
     for (i in names(object)) {
 
@@ -181,10 +210,10 @@ read.sav <- function(file, use.value.labels = FALSE, use.missings = TRUE, format
 
     #...................
     ### Factors ####
-    if (isTRUE(any(unlist(object.attributes) == "labels") && isTRUE(use.value.labels))) {
+    if (isTRUE(any(unlist(object.attributes) == "labels") && use.value.labels)) {
 
       var.labels.na <- NULL
-      for (i in which(vapply(object.attributes, function(y) any(y == "labels"), FUN.VALUE = logical(1)))) {
+      for (i in which(vapply(object.attributes, function(y) any(y == "labels"), FUN.VALUE = logical(1L)))) {
 
         # Labels
         labels <- attributes(object[, i])$labels
@@ -209,8 +238,7 @@ read.sav <- function(file, use.value.labels = FALSE, use.missings = TRUE, format
 
       if (isTRUE(!is.null(var.labels.na))) {
 
-        warning(paste0("Value labels are not specified for all values of the variable: ",
-                       paste(colnames(object[, var.labels.na, drop = FALSE]), collapse = ", ")), call. = FALSE)
+        warning(paste0("Value labels are not specified for all values of the variable: ", paste(colnames(object[, var.labels.na, drop = FALSE]), collapse = ", ")), call. = FALSE)
 
       }
 
